/*
 * Ported from OLED_SSD1306.{h,c} written by 43oh (www.43oh.com) for the first generation OLED display
 */
 
// Chip Select
#define OLED_NCS	12  // PB4
#define OLED_NCS_MASK   0x10

// Software SPI
#define OLED_SCLK       13  // PB5
#define OLED_SCLK_MASK  0x20
#define OLED_MOSI       11  // PB3
#define OLED_MOSI_MASK  0x08

#define SET_PORTB_BIT(n)  PORTB = PORTB | n
#define CLR_PORTB_BIT(n)  PORTB = PORTB & ~n

#define SSD1306_LCDWIDTH 128
#define SSD1306_LCDHEIGHT 64

#define SSD1306_SETCONTRAST 0x81
#define SSD1306_DISPLAYALLON_RESUME 0xA4
#define SSD1306_DISPLAYALLON 0xA5
#define SSD1306_NORMALDISPLAY 0xA6
#define SSD1306_INVERTDISPLAY 0xA7
#define SSD1306_DISPLAYOFF 0xAE
#define SSD1306_DISPLAYON 0xAF
#define SSD1306_SETDISPLAYOFFSET 0xD3
#define SSD1306_SETCOMPINS 0xDA
#define SSD1306_SETVCOMDETECT 0xDB
#define SSD1306_SETDISPLAYCLOCKDIV 0xD5
#define SSD1306_SETPRECHARGE 0xD9
#define SSD1306_SETMULTIPLEX 0xA8
#define SSD1306_SETLOWCOLUMN 0x00
#define SSD1306_SETHIGHCOLUMN 0x10
#define SSD1306_SETSTARTLINE 0x40
#define SSD1306_MEMORYMODE 0x20
#define SSD1306_COMSCANINC 0xC0
#define SSD1306_COMSCANDEC 0xC8
#define SSD1306_SEGREMAP 0xA0
#define SSD1306_CHARGEPUMP 0x8D
#define SSD1306_EXTERNALVCC 0x1
#define SSD1306_SWITCHCAPVCC 0x2



/* Function    : SSD1306Init()
 * Description : Initializes the OLED display
 * Input       : None
 * Output      : None
 */
void SSD1306Init( void )
{
   // Assumes display is reset by hardware reset
   
   // Configure our software SPI interface
   pinMode(OLED_NCS, OUTPUT);
   digitalWrite(OLED_NCS, HIGH);
   pinMode(OLED_SCLK, OUTPUT);
   digitalWrite(OLED_SCLK, LOW);
   pinMode(OLED_MOSI, OUTPUT);
   digitalWrite(OLED_MOSI, LOW);

   // Initialize panel
   SSD1306SendCommand(0xae);//--turn off oled panel 
   SSD1306SendCommand(0x00);//---set low column address 
   SSD1306SendCommand(0x10);//---set high column address 
   SSD1306SendCommand(0x40);//--set start line address 
   SSD1306SendCommand(0x81);//--set contrast control register 
   SSD1306SendCommand(0xcf); 
   SSD1306SendCommand(0xa1);//--set segment re-map 95 to 0 
   SSD1306SendCommand(0xa6);//--set normal display 
   SSD1306SendCommand(0xa8);//--set multiplex ratio(1 to 64) 
   SSD1306SendCommand(0x3f);//--1/64 duty 
   SSD1306SendCommand(0xd3);//-set display offset 
   SSD1306SendCommand(0x00);//-not offset 
   SSD1306SendCommand(0xd5);//--set display clock divide ratio/oscillator frequency 
   SSD1306SendCommand(0x80);//--set divide ratio 
   SSD1306SendCommand(0xd9);//--set pre-charge period 
   SSD1306SendCommand(0xf1); 
   SSD1306SendCommand(0xda);//--set com pins hardware configuration 
   SSD1306SendCommand(0x12); 
   SSD1306SendCommand(0xdb);//--set vcomh 
   SSD1306SendCommand(0x40); 
   SSD1306SendCommand(0x8d);//--set Charge Pump enable/disable 
   SSD1306SendCommand(0x14);//--set(0x10) disable 
   SSD1306SendCommand(0xaf);//--turn on oled panel

   Fill_RAM(0x00);				// Clear Screen
}


/* Function    : SSD1306SendCommand( char c )
 * Description : Sends a command to the OLED display
 * Input       : Command
 * Output      : None
 */
void SSD1306SendCommand( uint8_t c )
{ 
   // Bit 8: D/C# = 0 (Command)
   uint16_t data = c;
   uint8_t i = 9;

   // Output 9 bits
   //digitalWrite(OLED_NCS, LOW);
   CLR_PORTB_BIT(OLED_NCS_MASK);
   while (i--) {
     if (data & 0x0100) {
       //digitalWrite(OLED_MOSI, HIGH);
       SET_PORTB_BIT(OLED_MOSI_MASK);
     } else {
       //digitalWrite(OLED_MOSI, LOW);
       CLR_PORTB_BIT(OLED_MOSI_MASK);
     }
     //digitalWrite(OLED_SCLK, HIGH);
     SET_PORTB_BIT(OLED_SCLK_MASK);
     data = data << 1;
     //digitalWrite(OLED_SCLK, LOW);
     CLR_PORTB_BIT(OLED_SCLK_MASK);
   }
   //digitalWrite(OLED_NCS, HIGH);
   SET_PORTB_BIT(OLED_NCS_MASK);
}


/* Function    : SSD1306SendData( char c )
 * Description : Sends data to the OLED display
 * Input       : Data
 * Output      : None
 */
void SSD1306SendData( uint8_t c )
{
   // Bit 8: D/C# = 1 (Data)
   uint16_t data = c | 0x0100;
   uint8_t i = 9;

   // Output 9 bits
   //digitalWrite(OLED_NCS, LOW);
   CLR_PORTB_BIT(OLED_NCS_MASK);
   while (i--) {
     if (data & 0x0100) {
       //digitalWrite(OLED_MOSI, HIGH);
       SET_PORTB_BIT(OLED_MOSI_MASK);
     } else {
       //digitalWrite(OLED_MOSI, LOW);
       CLR_PORTB_BIT(OLED_MOSI_MASK);
     }
     //digitalWrite(OLED_SCLK, HIGH);
     SET_PORTB_BIT(OLED_SCLK_MASK);
     data = data << 1;
     //digitalWrite(OLED_SCLK, LOW);
     CLR_PORTB_BIT(OLED_SCLK_MASK);
   }
   //digitalWrite(OLED_NCS, HIGH);
   SET_PORTB_BIT(OLED_NCS_MASK);
}


/* Function    : Fill_RAM( unsigned char c )
 * Description : Fills the OLED with one particular data
 * Input       : Data
 * Output      : None
 */
void Fill_RAM(unsigned char Data)
{
   unsigned char i,j;

   for(i=0;i<8;i++)
   {
      Set_Start_Page(i);
      Set_Start_Column(0x00);

      for(j=0;j<128;j++) {
        SSD1306SendData(Data);
      }
   }
}


/* Function    : Fill_RAM_PAGE
 * Description : Fills an OLED page with a particular data
 * Input       : page no.[0-7], Data
 * Output      : None
 */
void Fill_RAM_PAGE(unsigned char page, unsigned char Data)
{
   unsigned char j;

   Set_Start_Page(page);
   Set_Start_Column(0x00);

   for(j=0;j<128;j++) {
     SSD1306SendData(Data);
   }
}


/* Function    : Fill_RAM_FONT_SMALL
 * Description : Writes a charcter on the OLED display at a column, row.
 *               with a small font.
 * Input       : character, column, row
 * Output      : None
 */
void Fill_RAM_FONT_SMALL(  unsigned char number, 
                           unsigned char column,
                           unsigned char row )
{
  int j=0;
  Set_Start_Page(row);
  Set_Start_Column(column);	

  for(j=0;j<SMALL_FONT_WIDTH;j++) {
    SSD1306SendData(pgm_read_byte_near(data_table_SMALL + j+(number*SMALL_FONT_WIDTH)));
    //SSD1306SendData(data_table_SMALL[j+(number*SMALL_FONT_WIDTH)]);
  }
}


/* Function    : Fill_RAM_FONT
 * Description : Writes a charcter on the OLED display at a column, row.
 *               with a large font.
 * Input       : character, column, row, span and width
 * Output      : None
 */
void Fill_RAM_FONT(  unsigned char number, 
                     unsigned char column,
                     unsigned char row,
                     unsigned char span, 
                     unsigned char width )
{
  int i,j,test; 
  int adder=0;
  int jump = 0;

  jump = (number - 32);
  for(i=row;i>=row-span+1;i--) {
    Set_Start_Page(i);
    Set_Start_Column(column);

    for(j=0;j<width;j++) {
      test = j+(adder*width)+(jump*28);
      SSD1306SendData(pgm_read_byte_near(data_table_LARGE + test));
      //SSD1306SendData(data_table_LARGE[test]);
     }
  adder++;
  }
}


/* Function    : Set_Start_Column
 * Description : Sets a start column to start writing at.
 * Input       : column[0-127]
 * Output      : None
 */
void Set_Start_Column(unsigned char d)
{
  // Set Lower Column Start Address for Page Addressing Mode. Default => 0x00
  SSD1306SendCommand(0x00+d%16);		

  // Set Higher Column Start Address for Page Addressing Mode. Default => 0x10
  SSD1306SendCommand(0x10+d/16);		
}


/* Function    : Set_Start_Page
 * Description : Sets a start page to start writing at.
 * Input       : column[0-7]
 * Output      : None
 */
void Set_Start_Page(unsigned char d)
{
  // Set Page Start Address for Page Addressing Mode. Default => 0xB0 (0x00)
  SSD1306SendCommand(0xB0|d);								
}

